/*------------------------------------------------------------------------------
 bin/djv_ls/app_cmdln.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include "app.h"
#include "etc.h"

namespace djv_ls {

void App::cmdln(Str & in) throw (Error)
{
  app::App::cmdln(in);
  if (exit() != djv_base::app::EXIT_DEFAULT) return;
  
  Str arg;
try
{
  while (! in.empty())
  {
    in >> arg;

    // Options.

    if ("-seq" == arg || "-q" == arg) in >> _seq;
    else if ("-recurse" == arg || "-r" == arg) _recurse = true;
    else if ("-hidden" == arg) _hidden = true;
    else if ("-filter" == arg || "-f" == arg) in >> _filter;
    else if ("-columns" == arg || "-c" == arg) in >> _columns;
    
    // Sorting.
    
    else if ("-sort" == arg || "-s" == arg) in >> _sort;
    else if ("-sort_reverse" == arg || "-sr" == arg) _sort_reverse = true;
    else if ("-sort_dir" == arg) in >> _sort_dir;
    
    // Arguments.
    
    else _input += arg;
  }
}
catch (Str) { throw Error(arg); }
}

Str App::cmdln_help() const
{
  return str::Format(label_cmdln_help).
    arg(file::label_seq_compress(), ", ").
    arg(str::label(_seq)).
    arg(file::label_sort(), ", ").
    arg(str::label(_sort)).
    arg(str::label_bool(), ", ").
    arg(str::label(_sort_dir)).
    arg(app::App::cmdln_help());
}

const Str label_cmdln_help =
"\n"
"djv_ls\n"
"------\n"
"\n"
"Command line utility for directory listing.\n"
"\n"
"Output\n"
"------\n"
"\n"
"Format:\n"
"\n"
"(name) (size) (user) (type) (permissions) (time)\n"
"\n"
"Type:\n"
"* file - Rgular file.\n"
"* seq - File sequence.\n"
"* dir - Directory.\n"
"\n"
"Permissions:\n"
"* r - Readable.\n"
"* w - Writable.\n"
"* x - Executable.\n"
"\n"
"Example:\n"
"\n"
"el_cerrito_bart.1k.tiff        2.23MB darby File rw Mon Jun 12 21:21:55 2006\n"
"richmond_train.2k.tiff         8.86MB darby File rw Mon Jun 12 21:21:58 2006\n"
"fishpond.1-749.png            293.17MB darby Seq rw Thu Aug 17 16:47:43 2006\n"
"\n"
"Usage\n"
"-----\n"
"\n"
"* djv_ls [file|directory]... [option]...\n"
"\n"
"Options\n"
"-------\n"
"\n"
"* -seq, -q (value)\n"
"  File sequencing: %%. Value: %%.\n"
"\n"
"* -recurse, -r\n"
"  Descend into sub-directories.\n"
"\n"
"* -hidden\n"
"  Show hidden files.\n"
"\n"
"* -filter, -f (value)\n"
"  Add a filter using shell globbing (e.g., *, ?).\n"
"\n"
"* -columns, -c (value)\n"
"  Number of columns for formatting. A value of zero disables formatting.\n"
"\n"
"Sorting:\n"
"\n"
"* -sort, -s (value)\n"
"  Sort: %%. Value: %%.\n"
"\n"
"* -sort_reverse, -sr\n"
"  Reverse sort order.\n"
"\n"
"* -sort_dir (value)\n"
"  Sort directories first: %%. Value: %%.\n"
"\n"
"General Options\n"
"---------------\n"
"\n"
"%%"
"\n"
"Examples\n"
"--------\n"
"\n"
"List the current directory:\n"
"> djv_ls\n"
"\n"
"List multiple directories:\n"
"> djv_ls ~/movies ~/pictures\n"
"\n"
"Sort by time, with the most recent first:\n"
"> djv_ls -sort time -sort_reverse\n"
"\n"
"Show only files ending with .tiff or .tif (note use of double-quotes):\n"
"> djv_ls -filter \"*.tiff\" -filter \"*.tif\"\n"
"\n";

}

