/*------------------------------------------------------------------------------
 bin/djv_view/file_prefs.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include "app.h"
#include "file.h"
#include "image_cache.h"
#include <djv_ui/file.h>
#include <djv_ui/prefs.h>

namespace djv_view {
namespace file {

Prefs::Prefs() :
  recent_sig(this),
  proxy_sig(this),
  cache_sig(this),
  _seq_auto(true),
  _cmdln_combine(false),
  _proxy(djv_image::data::PROXY(0)),
  _cache(false),
  _cache_size(image_cache::default_size()[0]),
  _cache_display(true)
{
  djv_ui::prefs::Prefs prefs(djv_ui::prefs::prefs(), "file");

  djv_ui::prefs::get(&prefs, "seq_auto", &_seq_auto);
  djv_ui::prefs::get(&prefs, "cmdln_combine", &_cmdln_combine);
  djv_ui::prefs::get(&prefs, "recent", &_recent);
  djv_ui::prefs::get(&prefs, "proxy", &_proxy);
  djv_ui::prefs::get(&prefs, "cache", &_cache);
  djv_ui::prefs::get(&prefs, "cache_size", &_cache_size);
  djv_ui::prefs::get(&prefs, "cache_display", &_cache_display);
  
  image_cache::global()->max(_cache_size);
}

Prefs::~Prefs()
{
  djv_ui::prefs::Prefs prefs(djv_ui::prefs::prefs(), "file");
  
  djv_ui::prefs::set(&prefs, "seq_auto", _seq_auto);
  djv_ui::prefs::set(&prefs, "cmdln_combine", _cmdln_combine);
  djv_ui::prefs::set(&prefs, "recent", _recent);
  djv_ui::prefs::set(&prefs, "proxy", _proxy);
  djv_ui::prefs::set(&prefs, "cache", _cache);
  djv_ui::prefs::set(&prefs, "cache_size", _cache_size);
  djv_ui::prefs::set(&prefs, "cache_display", _cache_display);
}

void Prefs::seq_auto(bool in)
{
  _seq_auto = in;
}

bool Prefs::seq_auto() const { return _seq_auto; }

void Prefs::cmdln_combine(bool in)
{
  _cmdln_combine = in;
}

bool Prefs::cmdln_combine() const { return _cmdln_combine; }

void Prefs::recent(const Str & in)
{
  DJV_DBG_OBJ("Prefs::recent");
  DJV_DBG_PRN("in = " << in);
  
  djv_ui::file::recent(in, &_recent);
  
  recent_sig.emit(_recent);
}

const List<Str> & Prefs::recent() const { return _recent; }

void Prefs::proxy(djv_image::data::PROXY in)
{
  if (in == _proxy) return;
  
  _proxy = in;
  
  proxy_sig.emit(_proxy);
}

djv_image::data::PROXY Prefs::proxy() const { return _proxy; }

void Prefs::cache(bool in)
{
  if (in == _cache) return;
  
  _cache = in;
  
  if (! _cache) image_cache::global()->del();
  cache_sig.emit(_cache);
}

bool Prefs::cache() const { return _cache; }

void Prefs::cache_size(int in)
{
  if (in == _cache_size) return;
  
  DJV_DBG_OBJ("Prefs::cache_size");
  DJV_DBG_PRN("in = " << in);
  
  _cache_size = in;
  
  image_cache::global()->max(_cache_size);
}

int Prefs::cache_size() const { return _cache_size; }

void Prefs::cache_display(bool in)
{
  if (in == _cache_display) return;
  
  _cache_display = in;
  
  app()->window_update();
}

bool Prefs::cache_display() const { return _cache_display; }

}
}

