/*------------------------------------------------------------------------------
 bin/djv_view/image_cache.h
 
 Image Cache
 -----------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_VIEW_IMAGE_CACHE_H
#define DJV_VIEW_IMAGE_CACHE_H

#include <djv_image/image.h>

namespace djv_view {
namespace image_cache {
using namespace djv_base;

//------------------------------------------------------------------------------
// Image Reference

class Ref
{
public:

  Ref(djv_image::Image *, const void * key, int frame);
  
  // Value.
  
  djv_image::Image * get();

  // Key.
  
  void key(const void *);
  const void * key() const;
  int frame() const;
  
  // Count.

  void ref();
  void ref_del();
  int ref_count() const;
  
private:

  std::auto_ptr<djv_image::Image> _image;
  const void * _key;
  int _frame;
  int _ref_count;
};

//------------------------------------------------------------------------------
// Image Cache

class Cache
{
public:

  Cache();
  ~Cache();

  // Value.
  
  Ref * create(djv_image::Image *, const void * key, int frame);
  Ref * get(const void * key, int frame);
  
  void max(int megabytes);
  int max() const;
  int size() const;
  
  void del(const void * key);
  void del();
  
  List<int> frame_list(const void * key);

  // Debugging.
  
  void dbg();

private:

  void purge();
  
  List<Ref *> _ref_list;
  uint64_t _cache_max, _cache_size;
};

//------------------------------------------------------------------------------
// Defaults

const List<int> & default_size();

//------------------------------------------------------------------------------
// Labels

const List<Str> & label_size();

//------------------------------------------------------------------------------
// Global

Cache * global();

}
}

#endif

