/*------------------------------------------------------------------------------
 bin/djv_view/image_histogram_dialog.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include "app.h"
#include "image_histogram.h"
#include <djv_ui/app.h>
#include <djv_ui/button.h>
#include <djv_ui/choice.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/prefs.h>
#include <djv_ui/text.h>

namespace djv_view {
namespace image_histogram {

struct Dialog::_Dialog
{
  image_histogram::Widget * widget;
  djv_ui::choice::Widget * size;
  djv_ui::text::Display * min, * max;
  djv_ui::button::Push * close;
};

Dialog::Dialog() :
  Base(dialog_title),
  _view(0),
  _size(djv_image::gl_data::HISTOGRAM_256),
  _p(new _Dialog)
{
  using namespace djv_ui;
  
  // Widgets.
  
  _p->widget = new image_histogram::Widget;
  
  _p->size = new choice::Widget(djv_image::gl_data::label_histogram());
  label::Widget * size_label = new label::Widget(dialog_size);
  
  _p->min = new text::Display;
  label::Widget * min_label = new label::Widget(dialog_min);
  _p->max = new text::Display;
  label::Widget * max_label = new label::Widget(dialog_max);
  
  _p->close = new button::Push(djv_ui::dialog::label_close);

  // Layout.
  
  layout::group(List<layout::Item *>() << min_label << max_label);
  
  Layout_V * layout = new Layout_V(this);
  
  layout->add(_p->widget);
  layout->stretch(_p->widget);
  
  Layout_H * layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add(min_label);
  layout_h->add(_p->min);  
  layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add(max_label);
  layout_h->add(_p->max);
  
  layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add(size_label);
  layout_h->add(_p->size);
  layout_h->add_spacer(-1, true);
  layout_h->add(_p->close);
  
  // Preferences.

  prefs::Prefs prefs(prefs::prefs(), "image_histogram");
  prefs::get(&prefs, "size", &_size);

  // Initialize.

  widget_update();

  size(size_hint());
  
  // Callbacks.
  
  _p->size->sig.set(this, size_callback);
  _p->close->sig.set(this, close_callback);
}

Dialog::~Dialog()
{}

void Dialog::del()
{
  using namespace djv_ui;
  
  Base::del();

  prefs::Prefs prefs(prefs::prefs(), "image_histogram");
  prefs::set(&prefs, "size", _size);
}

void Dialog::set(const view::Widget * in)
{
  _view = in;
  
  widget_update();
}

void Dialog::pick(const view::Widget * in)
{
  if (in == _view || ! shown() || ! visible()) return;
  
  set(in);
}

void Dialog::update(const view::Widget * in)
{
  if (in != _view || ! shown() || ! visible()) return;
  
  widget_update();
}

void Dialog::del(const view::Widget * in)
{
  if (in != _view) return;
  
  _view = 0;
}

void Dialog::show()
{
  _p->close->take_focus();
  
  djv_ui::dialog::Base::show();
}

void Dialog::size_callback(int in)
{
  _size = djv_image::gl_data::HISTOGRAM(in);
  
  widget_update();
}

void Dialog::close_callback(bool)
{
  hide();
}

void Dialog::widget_update()
{
  if (! _view) return;
  const djv_image::Data * data = _view->get();
  if (! data) return;
  
  DJV_DBG_OBJ("Dialog::widget_update");
  
  callbacks(false);
  
  try
  {
    djv_image::Data tmp(djv_image::Info(data->size(), data->pixel()));
    djv_image::gl_data::copy(*data, &tmp, _view->option());
    djv_image::gl_data::histogram(tmp, &_histogram, _size, &_min, &_max);
  }
  catch (Error in)
  {
    app()->error(in);
  }
  
  _p->widget->set(&_histogram, _min, _max);
  _p->size->set(_size);
  _p->min->set(str::label(_min));
  _p->max->set(str::label(_max));
  
  callbacks(true);
  
  Base::update();
}

const Str
  dialog_title = "Image Histogram",
  dialog_size = "Size:",
  dialog_min = "Min:",
  dialog_max = "Max:";

}
}

