/*------------------------------------------------------------------------------
 bin/djv_view/image_info.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include "image_info.h"
#include <djv_ui/button.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/prefs.h>
#include <djv_ui/text.h>

namespace djv_view {
namespace image_info {

struct Dialog::_Dialog
{
  djv_ui::text::Display * name;
  djv_ui::text::Display * size;
  djv_ui::text::Display * pixel;
  djv_ui::text::Multiline_Display * tag;
  djv_ui::button::Push * close;
};

Dialog::Dialog() :
  Base(dialog_title),
  _obj(0),
  _p(new _Dialog)
{
  using namespace djv_ui;
  
  // Widgets.

  _p->name = new text::Display;
  label::Widget * name_label = new label::Widget(dialog_name);
  _p->size = new text::Display;
  label::Widget * size_label = new label::Widget(dialog_size);
  _p->pixel = new text::Display;
  label::Widget * pixel_label = new label::Widget(dialog_pixel);
  _p->tag = new text::Multiline_Display;
  label::Widget * tag_label = new label::Widget(dialog_tag);

  _p->close = new button::Push(djv_ui::dialog::label_close);
  
  // Layout.
  
  layout::group(List<layout::Item *>() <<
    name_label <<
    size_label <<
    pixel_label);
  
  Layout_V * layout = new Layout_V(this);
  
  Layout_H * layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add(name_label);
  layout_h->add(_p->name);
  
  layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add(size_label);
  layout_h->add(_p->size);
  
  layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add(pixel_label);
  layout_h->add(_p->pixel);
  
  layout->add(tag_label);
  layout->add(_p->tag);
  layout->stretch(_p->tag);
  
  layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add_stretch();
  layout_h->add(_p->close);
  
  // Initialize.
  
  size(Vec2i(300, 300));
  
  // Callbacks.
 
  _p->close->sig.set(this, close_callback);
}

Dialog::~Dialog()
{}

void Dialog::set(const void * in, const Info & info)
{
  _obj = in;
  _info = info;
  
  widget_update();
}

void Dialog::pick(const void * in, const Info & info)
{
  if (in == _obj || ! shown() || ! visible()) return;
  
  set(in, info);
}

void Dialog::update(const void * in, const Info & info)
{
  if (in != _obj || ! shown() || ! visible()) return;
  
  _info = info;
  
  widget_update();
}

void Dialog::del(const void * in)
{
  if (in != _obj) return;
  
  _obj = 0;
}

void Dialog::show()
{
  _p->close->take_focus();
  
  djv_ui::dialog::Base::show();
}

void Dialog::close_callback(bool)
{
  hide();
}

void Dialog::widget_update()
{
  callbacks(false);
  
  _p->name->set(_info.info.name);

  _p->size->set(str::Format("%%x%%:%%").
    arg(_info.info.size.x).
    arg(_info.info.size.y).
    arg(vec::aspect(_info.info.size)));
  
  _p->pixel->set(str::Format("%%").
    arg(djv_image::pixel::label_pixel()[_info.info.pixel]));

  Str tmp;
  const List<Str> key_list = _info.tag.key_list();
  for (uint i = 0; i < key_list.size(); ++i)
    tmp += str::Format("%% = %%\n").
      arg(key_list[i]).arg(_info.tag[key_list[i]]);
  _p->tag->set(tmp);

  callbacks(true);
}

const Str
  dialog_title = "Image Information",
  dialog_name = "Name:",
  dialog_size = "Size:",
  dialog_pixel = "Pixel:",
  dialog_tag = "Tags:";

}
}

