/*-----------------------------------------------------------------------------
 bin/djv_view/image_magnify.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include "image_magnify.h"
#include "view.h"
#include <djv_ui/button.h>
#include <djv_ui/choice.h>
#include <djv_ui/group.h>
#include <djv_ui/image_view.h>
#include <djv_ui/label.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/prefs.h>

namespace djv_view {
namespace image_magnify {

struct Dialog::_Dialog
{
  djv_ui::image_view::Widget * widget;
  djv_ui::choice::Widget * zoom;
  djv_ui::button::Push * close;
};

Dialog::Dialog() :
  Base(dialog_title),
  _view(0),
  _zoom(ZOOM_200),
  _p(new _Dialog)
{
  using namespace djv_ui;
  
  // Widgets.

  _p->widget = new image_view::Widget;
  
  _p->zoom = new choice::Widget(label_zoom());
  
  _p->close = new button::Push(djv_ui::dialog::label_close);
  
  group::Frame * frame = new group::Frame;
  frame->frame(FL_THIN_DOWN_BOX);

  // Layout.
  
  Layout_V * layout = new Layout_V(this);
  
  layout->add(frame);
  layout->stretch(frame);
  Layout_V * layout_v = new Layout_V(frame);
  layout_v->margin(0);
  layout_v->add(_p->widget);
  
  Layout_H * layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add(_p->zoom);
  layout_h->add_spacer(-1, true);
  layout_h->add(_p->close);
  
  // Preferences.
  
  prefs::Prefs prefs(prefs::prefs(), "image_magnify");
  prefs::get(&prefs, "zoom", &_zoom);

  // Initialize.
  
  widget_update();

  size(max(size_hint(), Vec2i(300, 300)));
  
  // Callbacks.

  _p->zoom->sig.set(this, zoom_callback);
  _p->close->sig.set(this, close_callback);
}

Dialog::~Dialog()
{}

void Dialog::del()
{
  using namespace djv_ui;
  
  Base::del();

  prefs::Prefs prefs(prefs::prefs(), "image_magnify");
  prefs::set(&prefs, "zoom", _zoom);
}

void Dialog::set(const view::Widget * in, const Vec2i & pick)
{
  _view = in;
  _pick = pick;
  
  widget_update();
}

void Dialog::pick(const view::Widget * in, const Vec2i & pick)
{
  if (! shown() || ! visible()) return;
  
  set(in, pick);
}

void Dialog::update(const view::Widget * in)
{
  if (in != _view || ! shown() || ! visible()) return;
  
  widget_update();
}

void Dialog::del(const view::Widget * in)
{
  if (in != _view) return;
  
  _view = 0;
  
  _p->widget->set(0);
  _p->widget->redraw();
}

void Dialog::show()
{
  _p->close->take_focus();

  djv_ui::dialog::Base::show();
}

void Dialog::zoom_callback(int in)
{
  _zoom = ZOOM(in);
  
  widget_update();
}

void Dialog::close_callback(bool) { hide(); }

void Dialog::widget_update()
{
  if (! _view) return;

  DJV_DBG_OBJ("Dialog::widget_update");
  
  callbacks(false);

  const double zoom = math::pow(2, _zoom);
  DJV_DBG_PRN("zoom = " << zoom);
  
  _p->widget->set(_view->get());
  _p->widget->view(
    -vec::floor<double, int>(Vec2f(_pick) * zoom) +
    _p->widget->geom().size / 2);
  _p->widget->zoom(_view->zoom() * zoom);
  _p->widget->option(_view->option());
  _p->widget->redraw();
  
  _p->zoom->set(_zoom);

  callbacks(true);
}

DJV_STR_OPERATOR_SET(ZOOM, label_zoom())
DJV_STR_OPERATOR_GET(ZOOM, label_zoom())

const List<Str> & label_zoom()
{
  static const List<Str> data = List<Str>() <<
    "100%" <<
    "200%" <<
    "400%" <<
    "800%" <<
    "1600%";
  return data;
}

const Str
  dialog_title = "Image Magnify";

}
}

