/*------------------------------------------------------------------------------
 bin/djv_view/window_prefs.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "app.h"
#include "window.h"
#include <djv_ui/prefs.h>

namespace djv_view {
namespace window {

Prefs::Prefs() :
  toolbar_sig(this),
  playbar_sig(this),
  _resize_fit(true),
  _resize_max(RESIZE_MAX_75),
  _toolbar(true),
  _playbar(true)
{
  djv_ui::prefs::Prefs prefs(djv_ui::prefs::prefs(), "window");
  djv_ui::prefs::get(&prefs, "geom", &_geom);
  djv_ui::prefs::get(&prefs, "resize_fit", &_resize_fit);
  djv_ui::prefs::get(&prefs, "resize_max", &_resize_max);
  djv_ui::prefs::get(&prefs, "toolbar", &_toolbar);
  djv_ui::prefs::get(&prefs, "playbar", &_playbar);
}

Prefs::~Prefs()
{
  djv_ui::prefs::Prefs prefs(djv_ui::prefs::prefs(), "window");
  djv_ui::prefs::set(&prefs, "geom", _geom);
  djv_ui::prefs::set(&prefs, "resize_fit", _resize_fit);
  djv_ui::prefs::set(&prefs, "resize_max", _resize_max);
  djv_ui::prefs::set(&prefs, "toolbar", _toolbar);
  djv_ui::prefs::set(&prefs, "playbar", _playbar);
}

void Prefs::geom(const Box2i & in)
{
  _geom = in;
}

const Box2i & Prefs::geom() const { return _geom; }

void Prefs::resize_fit(bool in)
{
  _resize_fit = in;
}

bool Prefs::resize_fit() const { return _resize_fit; }

void Prefs::resize_max(RESIZE_MAX in)
{
  _resize_max = in;
}

RESIZE_MAX Prefs::resize_max() const { return _resize_max; }

void Prefs::toolbar(bool in)
{
  if (in == _toolbar) return;
  
  _toolbar = in;
  
  toolbar_sig.emit(_toolbar);
}

bool Prefs::toolbar() const { return _toolbar; }

void Prefs::playbar(bool in)
{
  if (in == _playbar) return;
  
  _playbar = in;
  
  playbar_sig.emit(_playbar);
}

bool Prefs::playbar() const { return _playbar; }

}
}

