/*------------------------------------------------------------------------------
 include/djv_base/io.h

 File I/O
 --------
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IO_H
#define DJV_IO_H

#include <djv_base/base.h>

namespace djv_base {
namespace io {

//------------------------------------------------------------------------------
// Options

enum MODE
{
  READ,
  WRITE
};

//------------------------------------------------------------------------------
// I/O

class Io
{
public:

  Io();
  ~Io();

  // Open.
  
  void open(const Str &, MODE) throw (Error);
  void close();
  
  bool valid() const;

  // Set.

  void set(const void *, size_t, size_t word_size = 1) throw (Error);

  void set_8(const int8_t *, size_t) throw (Error);
  void set_u8(const uint8_t *, size_t) throw (Error);
  void set_16(const int16_t *, size_t) throw (Error);
  void set_u16(const uint16_t *, size_t) throw (Error);
  void set_32(const int32_t *, size_t) throw (Error);
  void set_u32(const uint32_t *, size_t) throw (Error);
  void set_f32(const float *, size_t) throw (Error);

  inline void set_8(const int8_t & in) throw (Error);
  inline void set_u8(const uint8_t & in) throw (Error);
  inline void set_16(const int16_t & in) throw (Error);
  inline void set_u16(const uint16_t & in) throw (Error);
  inline void set_32(const int32_t & in) throw (Error);
  inline void set_u32(const uint32_t & in) throw (Error);
  inline void set_f32(const float & in) throw (Error);
  
  // Get.
  
  void get(void *, size_t, size_t word_size = 1) throw (Error);
  
  void get_8(int8_t *, size_t = 1) throw (Error);
  void get_u8(uint8_t *, size_t = 1) throw (Error);
  void get_16(int16_t *, size_t = 1) throw (Error);
  void get_u16(uint16_t *, size_t = 1) throw (Error);
  void get_32(int32_t *, size_t = 1) throw (Error);
  void get_u32(uint32_t *, size_t = 1) throw (Error);
  void get_f32(float *, size_t = 1) throw (Error);

  // XXX Ill but fast.
  
  void readahead();

  inline const uint8_t * mmap_p() const;
  inline const uint8_t * mmap_end() const;

  // Position.

  void position(off_t) throw (Error);
  void seek(off_t) throw (Error);
  off_t position() const;
  
  // Size.
  
  inline size_t size() const;
  
  // Endian.

  void endian(bool);
  inline bool endian() const;

private:

  void position(off_t, bool seek) throw (Error);

#if defined(WIN32)
  HANDLE _f;
#else
  int _f;
#endif
  MODE _mode;
  size_t _size;
  bool _endian;
  void * _mmap;
  const uint8_t * _mmap_start, * _mmap_end, * _mmap_p;
  int _mmap_readahead;
};

//------------------------------------------------------------------------------
// Functions

void line(Io *, char *, int max_len = str::cstr_size) throw (Error);
void word(Io *, char *, int max_len = str::cstr_size) throw (Error);

List<Str> lines(const Str &);

//------------------------------------------------------------------------------
// Labels

extern const Str
  error,
  error_open,
  error_mmap,
  error_read,
  error_write,
  error_position;

}

//------------------------------------------------------------------------------
// Declarations

using io::Io;

}

#include <djv_base/io_.h>

#endif
