/*------------------------------------------------------------------------------
 include/djv_base/list_.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv_base {
namespace list {

template<typename T>
inline List<T>::List()
{}

template<typename T>
inline List<T>::List(const List<T> & in) :
  std::vector<T>(in)
{}

template<typename T>
inline List<T>::List(const List<T> & in, uint position, uint size) :
  std::vector<T>(size)
{
  const uint _size = in.size();
  for (uint i = 0, j = position; i < size && j < _size; ++i, ++j)
    (*this)[i] = in[j];
}

template<typename T>
inline List<T>::List(const T & in, uint size) :
  std::vector<T>(size, in)
{}

template<typename T>
inline List<T>::List(uint size) :
  std::vector<T>(size)
{}

template<typename T>
inline void List<T>::push_front(const T & in)
{
  List<T>::insert(List<T>::begin(), in);
}

template<typename T>
inline T List<T>::pop_front()
{
  const T out = List<T>::front();
  List<T>::erase(List<T>::begin());
  return out;
}

template<typename T>
inline T List<T>::pop_back()
{
  const T out = List<T>::back();
  std::vector<T>::pop_back();
  return out;
}

template<typename T>
inline void List<T>::append(const T & in)
{
  List<T>::push_back(in);
}

template<typename T>
inline void List<T>::append(const List<T> & in)
{
  for (uint i = 0; i < in.size(); ++i)
    List<T>::push_back(in[i]);
}

template<typename T>
inline List<T> & List<T>::operator += (const T & in)
{
  append(in);
  return *this;
}

template<typename T>
inline List<T> & List<T>::operator += (const List<T> & in)
{
  append(in);
  return *this;
}

template<typename T>
inline List<T> & List<T>::operator << (const T & in)
{
  List<T>::push_back(in);
  return *this;
}

template<typename T>
inline List<T> & List<T>::operator << (const List<T> & in)
{
  for (uint i = 0; i < in.size(); ++i)
    List<T>::push_back(in[i]);
  return *this;
}

template<typename T>
inline int find(const T & in, const List<T> & list)
{
  for (uint i = 0; i < list.size(); ++i)
    if (in == list[i]) return i;
  return -1;
}

}
}
