/*------------------------------------------------------------------------------
 include/djv_base/vec2_.h

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

namespace djv_base {
namespace vec {

template<typename T>
const int Vec<T, 2>::dimension = 2;

#define _VEC2_INIT \
  \
  x(Vec<T, 2>::e[0]), \
  y(Vec<T, 2>::e[1])

template<typename T>
inline Vec<T, 2>::Vec() :
  _VEC2_INIT
{
  x = y = 0;
}

template<typename T>
inline Vec<T, 2>::Vec(const Vec<T, 2> & in) :
  _VEC2_INIT
{
  x = in.x;
  y = in.y;
}

template<typename T>
inline Vec<T, 2>::Vec(T _x, T _y) :
  _VEC2_INIT
{
  x = _x;
  y = _y;
}

template<typename T>
inline Vec<T, 2>::Vec(T in) :
  _VEC2_INIT
{
  x = y = in;
}

template<typename T>
inline Vec<T, 2> & Vec<T, 2>::operator = (const Vec<T, 2> & in)
{
  if (&in != this) set(in);
  return *this;
}

template<typename T>
inline void Vec<T, 2>::set(const Vec<T, 2> & in)
{
  x = in.x;
  y = in.y;
}

template<typename T>
inline void Vec<T, 2>::set(T in)
{
  x = y = in;
}

template<typename T>
inline void Vec<T, 2>::zero()
{
  x = y = T(0);
}

#define _VEC2_OP(IN) \
  \
  template<typename T> \
  inline Vec<T, 2> & Vec<T, 2>::operator IN (const Vec<T, 2> & in) \
  { \
    x IN in.x; \
    y IN in.y; \
    return *this; \
  }
_VEC2_OP(+=)
_VEC2_OP(-=)
_VEC2_OP(*=)
_VEC2_OP(/=)

#define _VEC2_OP2(IN) \
  \
  template<typename T> \
  inline Vec<T, 2> & Vec<T, 2>::operator IN (T in) \
  { \
    x IN in; \
    y IN in; \
    return *this; \
  }
_VEC2_OP2(+=)
_VEC2_OP2(-=)
_VEC2_OP2(*=)
_VEC2_OP2(/=)

template<typename T>
inline Vec<T, 2>::operator Vec<int, 2> () const
{
  return Vec<int, 2>(int(x), int(y));
}

template<typename T>
inline Vec<T, 2>::operator Vec<double, 2> () const
{
  return Vec<double, 2>(double(x), double(y));
}

}
}

