/*------------------------------------------------------------------------------
 include/djv_image/io_ifl.h

 Image I/O - IFL
 ---------------
 
 Autodesk Image File List format.

 IFL is a file format for creating sequences or playlists of other image files.
 An IFL file simply consists of a list of image file names, one per line.
 
 Support:

 * Read only.

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_IO_IFL_H
#define DJV_IMAGE_IO_IFL_H

#include <djv_image/io.h>

namespace djv_image {
namespace io_ifl {

//------------------------------------------------------------------------------
// Information

static const Str name = "IFL";

static const List<Str> extension_list = List<Str>() << ".ifl";

//------------------------------------------------------------------------------
// Load Plugin

class Load : public io::Load
{
public:

  Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  void open(const File &, Info *, Tag *, Time *) throw (Error);
  void open(const File &, List<Info> *, Tag *, Time *) throw (Error);
  const Image * get(int frame, int layer, data::PROXY) throw (Error);

private:

  List<Str> _list;
  std::auto_ptr<io::Load> _load;
};

}
}

#endif

