/*------------------------------------------------------------------------------
 include/djv_image/io_rla.h

 Image I/O - RLA
 ---------------
 
 Wavefront image file format.

 Support:
 
 * Read-only.
 * Image data: L, LA, RGB, RGBA, 8-bit, 16-bit, 32-bit float.
 * File compression.

 References:
 
 * James D. Murray, William vanRyper, "Encyclopedia of Graphics File Formats,
   Second Edition"

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_IMAGE_IO_RLA_H
#define DJV_IMAGE_IO_RLA_H

#include <djv_image/io.h>

namespace djv_image {
namespace io_rla {

//------------------------------------------------------------------------------
// Information

static const Str name = "RLA";

static const List<Str> extension_list = List<Str>() << ".rla" << ".rpf";

//------------------------------------------------------------------------------
// Options

//------------------------------------------------------------------------------
// Load Plugin

class Load : public io::Load
{
public:

  Load();

  Plugin * copy() const;
  const Str & name() const;
  const List<Str> & extension_list() const;
  
  void open(const File &, Info *, Tag *, Time *) throw (Error);
  void open(const File &, List<Info> *, Tag *, Time *) throw (Error);
  const Image * get(int frame, int layer, data::PROXY) throw (Error);

private:

  void _open(const File &, List<Info> *) throw (Error);

  File _file;
  djv_base::Io _io;
  mem::Buffer<int32_t> _rle_offset;
  Image _image;
  Data _tmp;
};

//------------------------------------------------------------------------------
// Functions

void rle_load(
  djv_base::Io *,
  uint8_t *,
  int size,
  int channels,
  int bytes
) throw (Error);

void float_load(
  djv_base::Io *,
  uint8_t *,
  int size,
  int channels
) throw (Error);

void skip(djv_base::Io *) throw (Error);

//------------------------------------------------------------------------------
// Labels

}
}

#endif

