/*------------------------------------------------------------------------------
 include/djv_ui/group.h
 
 Group Widgets
 -------------

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_GROUP_H
#define DJV_UI_GROUP_H

#include <djv_ui/widget.h>

namespace djv_ui {
namespace group {

//------------------------------------------------------------------------------
// Frame Widget

class Frame : public widget::Widget
{
public:
  
  Frame(const Fl_Boxtype = FL_THIN_UP_FRAME);
  virtual ~Frame();
  
  // Frame.
  
  void frame(const Fl_Boxtype);
  void edges(bool left, bool right, bool top, bool bottom);
  const Fl_Boxtype & frame() const;
  const bool * edges() const;
  
  // Widget.
  
  virtual void draw();
  
  // Layout.

  virtual int frame_size() const;
  virtual Box2i frame_geom() const;
  
  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const;

private:

  Fl_Boxtype _frame;
  bool _edges [4];
};

//------------------------------------------------------------------------------
// Divider Widgets

class Divider : public widget::Widget
{
public:
  
  Divider(int margin = 0);
  virtual ~Divider() = 0;
  
  void margin(int);
  int margin() const;
  
private:

  int _margin;
};

class Divider_Horizontal : public Divider
{
public:
  
  Divider_Horizontal(int margin = 0);
  virtual ~Divider_Horizontal();
  
  virtual void draw();

  virtual void dirty();
};

class Divider_Vertical : public Divider
{
public:

  Divider_Vertical(int margin = 0);
  virtual ~Divider_Vertical();
  
  virtual void draw();

  virtual void dirty();
};

//------------------------------------------------------------------------------
// Group Box

class Box : public widget::Widget
{
public:

  Box(const Str & label = Str());
  virtual ~Box();
  virtual void del();
  
  // Label.

  void label(const Str &);
  const Str & label() const;
  
  // Widget.

  virtual void draw();
  
  // Layout.

  virtual void dirty();
  virtual void geom(const Box2i &);
  virtual const Box2i & geom() const;
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::group::Box"; }
  
private:
  
  Str _label;
  Vec2i _size_label;
};

}
}

#endif

