/*------------------------------------------------------------------------------
 include/djv_ui/widget.h
 
 Widget
 ------
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#ifndef DJV_UI_WIDGET_H
#define DJV_UI_WIDGET_H

#include <djv_ui/base.h>
#include <djv_ui/layout.h>
#include <FL/Fl_Group.H>

namespace djv_ui {
namespace widget {

//------------------------------------------------------------------------------
// Widget Base

class Base : public layout::Item, public Callback_Base
{
public:

  Base();
  virtual ~Base() = 0;
  virtual void del();
  
  // Widget.

  virtual void text_font(const Fl_Font &);
  virtual void text_color(const Fl_Color &);
  inline virtual const Fl_Font & text_font() const;
  inline virtual const Fl_Color & text_color() const;

  virtual void tooltip(const Str &);
  inline virtual const Str & tooltip() const;
  
  virtual int handle(int) = 0;
  virtual void show() = 0;
  virtual void draw() = 0;
  
  // Layout.

  virtual void dirty();
  virtual void layout(layout::Layout *);
  virtual void geom(const Box2i &) = 0;
  virtual void resize(int, int, int, int) = 0;
  inline virtual const Box2i & geom() const;
  inline virtual layout::Layout * layout() const;
  virtual int visible() const = 0;
  
  // Debugging.

  virtual Str dbg_name() const { return "djv::ui::widget::Base"; }

private:

  Fl_Font _text_font;
  Fl_Color _text_color;
  Str _tooltip;
  std::auto_ptr<layout::Layout> _layout;
  bool _del;
};

template<class T>
class Auto_Ptr
{
public:

  inline Auto_Ptr();
  inline Auto_Ptr(T *);
  inline ~Auto_Ptr();
  
  std::auto_ptr<T> p;
  
  inline T * get() const;
  inline T * operator -> () const;
};

//------------------------------------------------------------------------------
// Widget

class Widget : public Fl_Group, public Base
{
public:

  Widget();
  virtual ~Widget();
  
  // Widget.
  
  virtual void tooltip(const Str &);
  inline virtual const Str & tooltip() const;
  
  virtual int handle(int);
  virtual void show();
  virtual void draw();
  
  // Layout.

  virtual void dirty();
  virtual void geom(const Box2i &);
  inline virtual void resize(int, int, int, int);
  inline virtual const Box2i & geom() const;
  inline virtual int visible() const;

  virtual void size(layout::SIZE, layout::SIZE);
  virtual void align(layout::ALIGN, layout::ALIGN);
  inline const layout::SIZE * size() const;
  inline const layout::ALIGN * align() const;
  
  // Debugging.
  
  virtual Str dbg_name() const { return "djv::ui::widget::Widget"; }
  
private:
  
  mem::Buffer<char> _tooltip_cstr;
  bool _visible;
};

}
}

#include <djv_ui/widget_.h>

#endif
