/*------------------------------------------------------------------------------
 lib/djv_base/file_etc.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_base/file.h>
#include <djv_base/io.h>
#include <djv_base/math.h>

namespace djv_base {
namespace file {

bool exists(const File & in)
{
  DJV_DBG_OBJ("exists");
  DJV_DBG_PRN("in = " << in);
  
  List<Str> list;
  if (SEQ == in.type)
    list = seq_expand(in);
  else
    list += in;
  DJV_DBG_PRN("list = " << list);
  
  for (uint i = 0; i < list.size(); ++i)
    try
    {
      Io().open(list[i], io::READ);
      return true;
    }
    catch (Error) {}
  
  return false;;
}

Str frame_to_str(int in, int pad)
{
  char format [] = "%00d";
  static const int format_max = 9;
  format[2] = char('0' + math::min(pad, format_max));
  
  char tmp [str::cstr_size];
  const int size = SNPRINTF(tmp, str::cstr_size, format, in);
  return Str(tmp, size);
}

Str & operator << (Str & out, const Seq & in)
{
  return out << seq_to_str(in);
}

Str & operator >> (Str & in, Seq & out) throw (Str)
{
  Str tmp;
  in >> tmp;
  out = str_to_seq(tmp);
  return in;
}

DJV_STR_OPERATOR_SET(SORT, label_sort())
DJV_STR_OPERATOR_GET(SORT, label_sort())
DJV_STR_OPERATOR_SET(SEQ_COMPRESS, label_seq_compress())
DJV_STR_OPERATOR_GET(SEQ_COMPRESS, label_seq_compress())

const List<Str> & label_type()
{
  static const List<Str> data = List<Str>() <<
    "File" <<
    "Seq" <<
    "Dir";
  return data;
}

Str label_size(uint64_t in)
{
  if (in > mem::terabyte)
    return str::Format("%%TB").arg(in / double(mem::terabyte), 0, -2);
  else if (in > mem::gigabyte)
    return str::Format("%%GB").arg(in / double(mem::gigabyte), 0, -2);
  else if (in > mem::megabyte)
    return str::Format("%%MB").arg(in / double(mem::megabyte), 0, -2);
  else
    return str::Format("%%KB").arg(in / double(mem::kilobyte), 0, -2);
}

const Str & label_perm(int in)
{
  static const Str data [] =
  {
    "",
    "r",
    "w",
    "rw",
    "x",
    "rx",
    "wx",
    "rwx"
  };
  return data[in];
}

const List<Str> & label_sort()
{
  static const List<Str> data = List<Str>() <<
    "Name" <<
    "Type" <<
    "Size" <<
    "User" <<
    "Permissions" <<
    "Time";
  return data;
}

const List<Str> & label_seq_compress()
{
  static const List<Str> data = List<Str>() <<
    "Off" <<
    "Sparse" <<
    "Range";
  return data;
}

Dbg & operator << (Dbg & dbg, const Seq & in) { return dbg << str::label(in); }
Dbg & operator << (Dbg & dbg, const File & in) { return dbg << in.get(); }

}
}

