/*------------------------------------------------------------------------------
 lib/djv_base/file_filter.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_base/file.h>
#include <FL/filename.H>

namespace djv_base {
namespace file {

namespace {

inline bool path_is_relative(const Str & in)
{
  if ("." == in) ;
  else if (".." == in) ;
  else if (in.size() >= 2 && '.' == in[0] && ('/' == in[1] || '\\' == in[1])) ;
  else if (in.size() >= 3 && '.' == in[0] && '.' == in[1] &&
    ('/' == in[2] || '\\' == in[2])) ;
  else return false;
  return true;
}

}

void filter(
  List<File> * in,
  int filter,
  const List<Str> & glob_list
) {
  //DJV_DBG_OBJ("filter");
  //DJV_DBG_PRN("filter = " << filter);
  //DJV_DBG_PRN("globs = " << glob_list);
  
  List<File> & list = *in;
  
  uint i = 0;
  for (uint j = 0; j < list.size(); ++j)
  {
    const Str name = list[j].get(-1, false);
    //DJV_DBG_PRN("name = " << name);
    
    bool valid = true;
    
    if ((filter & FILTER_FILE) && (FILE == list[j].type || SEQ == list[j].type))
      valid = false;
    if ((filter & FILTER_DIR) && DIR == list[j].type)
      valid = false;
    if ((filter & FILTER_HIDDEN) && '.' == name[0] && ! path_is_relative(name))
      valid = false;
    
    if (glob_list.size())
    {
      uint k = 0;
      for (; k < glob_list.size(); ++k)
        if (fl_filename_match(name.c_str(), glob_list[k].c_str())) break;
      if (glob_list.size() == k)
        valid = false;
    }
    
    //DJV_DBG_PRN("valid = " << valid);
    if (valid)
    {
      if (i != j) list[i] = list[j];
      ++i;
    }
  }
  
  //DJV_DBG_PRN("size = " << i);
  list.resize(i);
}

}
}

