/*------------------------------------------------------------------------------
 lib/djv_image/data_etc.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_image/data.h>

namespace djv_image {
namespace data {

size_t bytes_scanline(const Info & in)
{
  return (in.size.x * pixel::bytes(in.pixel) * in.align) / in.align;
}

size_t bytes_data(const Info & in)
{
  return in.size.y * bytes_scanline(in);
}

void proxy_scale(const Data & in, Data * out, PROXY proxy)
{
  DJV_ASSERT(out);
  DJV_ASSERT(in.pixel() == out->pixel());
  
  DJV_DBG_OBJ("proxy_scale");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("out = " << *out);
  DJV_DBG_PRN("proxy = " << proxy);
  
  const int w = out->w(), h = out->h();
  const int proxy_scale = data::proxy_scale(proxy);
  const int bytes_pixel = out->bytes_pixel();
  
  for (int y = 0; y < h; ++y)
  
    // XXX Alignment issues?
    
    /*switch (bytes_pixel)
    {
      case 1:
      {
        const uint8_t * in_p = in.data(0, y * proxy_scale);
        uint8_t * out_p = out->data(0, y);
        for (int x = 0; x < w; ++x, in_p += proxy_scale, ++out_p)
          *out_p = *in_p;
      } break;
      
      case 2:
      {
        const uint16_t * in_p =
          reinterpret_cast<const uint16_t *>(in.data(0, y * proxy_scale));
        uint16_t * out_p =
          reinterpret_cast<uint16_t *>(out->data(0, y));
        for (int x = 0; x < w; ++x, in_p += proxy_scale, ++out_p)
          *out_p = *in_p;
      } break;
      
      case 4:
      {
        const uint32_t * in_p =
          reinterpret_cast<const uint32_t *>(in.data(0, y * proxy_scale));
        uint32_t * out_p =
          reinterpret_cast<uint32_t *>(out->data(0, y));
        for (int x = 0; x < w; ++x, in_p += proxy_scale, ++out_p)
          *out_p = *in_p;
      } break;
      
      case 8:
      {
        const uint64_t * in_p =
          reinterpret_cast<const uint64_t *>(in.data(0, y * proxy_scale));
        uint64_t * out_p =
          reinterpret_cast<uint64_t *>(out->data(0, y));
        for (int x = 0; x < w; ++x, in_p += proxy_scale, ++out_p)
          *out_p = *in_p;
      } break;
      
      default:*/
      {
        const uint8_t * in_p = in.data(0, y * proxy_scale);
        uint8_t * out_p = out->data(0, y);
        for (
          int x = 0;
          x < w;
          ++x, in_p += bytes_pixel * proxy_scale, out_p += bytes_pixel
        )
          switch (bytes_pixel)
          {
            case 16: out_p[15] = in_p[15];
            case 15: out_p[14] = in_p[14];
            case 14: out_p[13] = in_p[13];
            case 13: out_p[12] = in_p[12];
            case 12: out_p[11] = in_p[11];
            case 11: out_p[10] = in_p[10];
            case 10: out_p[9] = in_p[9];
            case 9: out_p[8] = in_p[8];
            case 8: out_p[7] = in_p[7];
            case 7: out_p[6] = in_p[6];
            case 6: out_p[5] = in_p[5];
            case 5: out_p[4] = in_p[4];
            case 4: out_p[3] = in_p[3];
            case 3: out_p[2] = in_p[2];
            case 2: out_p[1] = in_p[1];
            case 1: out_p[0] = in_p[0];
          }
      //} break;
  }
}

int proxy_scale(PROXY proxy)
{
  return proxy ? math::pow(2, int(proxy)) : 1;
}

Vec2i proxy_scale(const Vec2i & in, PROXY proxy)
{
  const int scale = proxy_scale(proxy);
  return Vec2i(
    math::ceil(in.x / double(scale)),
    math::ceil(in.y / double(scale)));
}

void planar_interleave(const Data & in, Data * out, PROXY proxy)
{
  DJV_ASSERT(out);
  DJV_ASSERT(in.pixel() == out->pixel());
  DJV_ASSERT(in.pixel() != pixel::RGB_U10);
  
  DJV_DBG_OBJ("planar_interleave");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("out = " << *out);
  DJV_DBG_PRN("proxy = " << proxy);
  
  const int w = out->w(), h = out->h();
  const int proxy_scale = data::proxy_scale(proxy);
  const int channels = out->channels();
  const int bytes_pixel = out->bytes_pixel();
  const int channel_bytes = pixel::channel_bytes(out->pixel());
  
  for (int c = 0; c < channels; ++c)
    for (int y = 0; y < h; ++y)
      /*switch (channel_bytes)
      {
        default:*/
        {
          const uint8_t * in_p = in.data() + (c * in.h() + y * proxy_scale) *
            in.w() * channel_bytes;
          uint8_t * out_p = out->data(0, y) + c * channel_bytes;
          for (
            int x = 0;
            x < w;
            ++x, in_p += channel_bytes * proxy_scale, out_p += bytes_pixel
          )
            switch (channel_bytes)
            {
              case 4: out_p[3] = in_p[3];
              case 3: out_p[2] = in_p[2];
              case 2: out_p[1] = in_p[1];
              case 1: out_p[0] = in_p[0];
            }
        //} break;
      }
}

void planar_deinterleave(const Data & in, Data * out)
{
  DJV_ASSERT(out);
  DJV_ASSERT(in.pixel() == out->pixel());
  DJV_ASSERT(in.pixel() != pixel::RGB_U10);
  
  DJV_DBG_OBJ("planar_deinterleave");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("out = " << *out);
  
  const int w = out->w(), h = out->h();
  const int channels = out->channels();
  const int bytes_pixel = out->bytes_pixel();
  const int channel_bytes = pixel::channel_bytes(out->pixel());
  
  for (int c = 0; c < channels; ++c)
    for (int y = 0; y < h; ++y)
      /*switch (channel_bytes)
      {
        default:*/
        {
          const uint8_t * in_p = in.data(0, y) + c * channel_bytes;
          uint8_t * out_p = out->data() + (c * h + y) * w * channel_bytes;
          for (
            int x = 0;
            x < w;
            ++x, in_p += bytes_pixel, out_p += channel_bytes
          )
            switch (channel_bytes)
            {
              case 4: out_p[3] = in_p[3];
              case 3: out_p[2] = in_p[2];
              case 2: out_p[1] = in_p[1];
              case 1: out_p[0] = in_p[0];
            }
        //} break;
      }
}

bool operator == (const Info & a, const Info & b)
{
  return
    a.size == b.size &&
    a.proxy == b.proxy &&
    a.pixel == b.pixel &&
    a.bgr == b.bgr &&
    a.mirror == b.mirror &&
    a.align == b.align &&
    a.endian == b.endian;
}

bool operator != (const Info & a, const Info & b)
{
  return ! (a == b);
}

DJV_STR_OPERATOR_SET(PROXY, label_proxy())
DJV_STR_OPERATOR_GET(PROXY, label_proxy())

const List<Str> & label_proxy()
{
  static const List<Str> data = List<Str>() <<
    "None" <<
    "1/2" <<
    "1/4" <<
    "1/8";
  return data;
}

Dbg & operator << (Dbg & dbg, PROXY in) { return dbg << str::label(in); }

Dbg & operator << (Dbg & dbg, const Info & in)
{
  return dbg <<
    "name:" << in.name << " " <<
    "size:" << in.size << " " <<
    "proxy:" << in.proxy << " " <<
    "pixel:" << in.pixel <<
    " bgr:" << in.bgr <<
    " mirror:" << in.mirror <<
    " align:" << in.align <<
    " endian:" << mem::label_endian()[in.endian];
}

Dbg & operator << (Dbg & dbg, const Data & in)
{
  return dbg << in.info();
}

}
}
