/*------------------------------------------------------------------------------
 lib/djv_image/gl.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_image/gl.h>
#include <djv_base/math.h>

namespace djv_image {
namespace gl {

void ortho(const Vec2i & size, const Vec2f & min_max)
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(0, size.x, 0, size.y, min_max.x, min_max.y);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
}

GLenum format(PIXEL in, bool bgr)
{
  switch (in)
  {
    case pixel::L_U8:
    case pixel::L_U16:
    case pixel::L_F16:
    case pixel::L_F32: return GL_LUMINANCE;
    case pixel::LA_U8:
    case pixel::LA_U16:
    case pixel::LA_F16:
    case pixel::LA_F32: return GL_LUMINANCE_ALPHA;
    case pixel::RGB_U8:
    case pixel::RGB_U16:
    case pixel::RGB_F16:
    case pixel::RGB_F32: return bgr ? GL_BGR : GL_RGB;
    case pixel::RGB_U10: return bgr ? GL_BGRA : GL_RGBA;
    case pixel::RGBA_U8:
    case pixel::RGBA_U16:
    case pixel::RGBA_F16: 
    case pixel::RGBA_F32: return bgr ? GL_BGRA : GL_RGBA;
    default: break;
  }
  return 0;
}

GLenum type(PIXEL in)
{
  switch (in)
  {
    case pixel::L_U8:
    case pixel::LA_U8:
    case pixel::RGB_U8:
    case pixel::RGBA_U8: return GL_UNSIGNED_BYTE;
    case pixel::RGB_U10: return GL_UNSIGNED_INT_10_10_10_2;
    case pixel::L_U16:
    case pixel::LA_U16:
    case pixel::RGB_U16:
    case pixel::RGBA_U16: return GL_UNSIGNED_SHORT;
    case pixel::L_F16:
    case pixel::LA_F16:
    case pixel::RGB_F16:
    case pixel::RGBA_F16:
      return GLEW_ARB_half_float_pixel ? GL_HALF_FLOAT_ARB : 0;
    case pixel::L_F32:
    case pixel::LA_F32:
    case pixel::RGB_F32:
    case pixel::RGBA_F32: return GL_FLOAT;
    default: break;
  }
  return 0;
}

GLenum texture()
{
  return 
    GLEW_ARB_texture_rectangle ?
    GL_TEXTURE_RECTANGLE_ARB :
    GL_TEXTURE_2D;
}

DJV_STR_OPERATOR_SET(PIPELINE, label_pipeline())
DJV_STR_OPERATOR_GET(PIPELINE, label_pipeline())

const List<Str> & label_pipeline()
{
  static const List<Str> data = List<Str>() <<
    "Fixed" <<
    "Program";
  return data;
}

Dbg & operator << (Dbg & dbg, PIPELINE in)
{
  return dbg << str::label(in);
}

Device::~Device()
{}

const Str & Device::vendor_str() const { return _vendor_str; }
const Str & Device::render_str() const { return _render_str; }
const Str & Device::version_str() const { return _version_str; }

Device * Global::device() const { return _device.get(); }

void Global::pipeline(PIPELINE in)
{
  _pipeline = in;
}

PIPELINE Global::pipeline() const { return _pipeline; }
  
}
}
