/*------------------------------------------------------------------------------
 lib/djv_image/io_cineon_save.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#define DJV_DBG

#include <djv_image/io_cineon.h>

namespace djv_image {
namespace io_cineon {

io::Plugin * Save::copy() const
{
  Save * out = new Save;
  out->_option = _option;
  return out;
}

const Str & Save::name() const { return io_cineon::name; }

const List<Str> & Save::extension_list() const
  { return io_cineon::extension_list; }

bool Save::option(const Str & in, Str * data)
{
try
{
  if (str::compare(in, option_list()[SAVE_COLOR])) *data >> _option.color;
  else if (str::compare(in, option_list()[SAVE_PRINT])) *data >> _option.print;
}
catch (Str) { return false; }
  return true;
}

Str Save::option(const Str & in) const
{
  Str out;
  if (str::compare(in, option_list()[SAVE_COLOR])) out << _option.color;
  else if (str::compare(in, option_list()[SAVE_PRINT])) out << _option.print;
  return out;
}

const List<Str> & Save::option_list() const { return label_save_option(); }

void Save::open(const File & in, const Info & info, const Tag &,
  const Time & time) throw (Error)
{
  DJV_DBG_OBJ("Save::open");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("info = " << info);
  
  _file = in;
  if (time.list.size() > 1) _file.type = file::SEQ;
  
  _info = Info();
  _info.size = info.size;
  _info.pixel = pixel::RGB_U10;
  _info.mirror.y = true;
  _info.endian = mem::MSB;
  _info.align = 4;
  DJV_DBG_PRN("info = " << _info);
  
  _image.set(_info);
}

void Save::_open(const File & in, const Tag & tag) throw (Error)
{
  DJV_DBG_OBJ("Save::_open");
  DJV_DBG_PRN("in = " << in);

  _io.open(in, djv_base::io::WRITE);
  
  info_save(&_io, in, _info, tag, _option.color);
}

void Save::set(const Image & in, int frame) throw (Error)
{
  DJV_DBG_OBJ("Save::set");
  DJV_DBG_PRN("in = " << in);
  
  // Setup.
  
  Profile profile;
  if (
    COLOR_PRINT == _option.color ||
    COLOR_AUTO == _option.color
  ) {
    DJV_DBG_PRN("profile");
    profile.type = profile::LUT;
    profile.lut = print_lut(_option.print);
  }
  
  // Open.
  
  _open(_file.get(frame), in.tag);
  
  // Write.
  
  const Data * p = &in;
  if (
    in.info() != _info ||
    in.profile.type != profile::RAW ||
    profile.type != profile::RAW
  ) {
    DJV_DBG_PRN("convert = " << _image);
    _image.zero();
    gl_data::Option option;
    option.profile = profile;
    gl_data::copy(*p, &_image, option);
    p = &_image;
  }
  
  _io.set(p->data(), p->bytes_data());

  info_update(&_io);
  _io.close();
}

}
}
