/*------------------------------------------------------------------------------
 lib/djv_ui/app_dialog.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/app.h>
#include <djv_ui/app_dialog.h>
#include <djv_ui/button.h>
#include <djv_ui/group.h>
#include <djv_ui/label.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/scroll.h>
#include <djv_ui/style.h>
#include <djv_ui/text.h>
#include <FL/Fl.H>
#include <FL/Fl_Pixmap.H>
#include <FL/fl_draw.H>
#include "durant.xpm"

namespace djv_ui {
namespace app_dialog {

struct Info::_Info
{
  text::Multiline_Display * widget;
  button::Push * close;
};

Info::Info() :
  dialog::Base(label_info),
  _p(new _Info)
{
  // Widgets.

  _p->widget = new text::Multiline_Display(
    str::join(str::format(app::global()->info(), 80), "\n"));
  
  _p->close = new button::Push(dialog::label_close);
    
  // Layout.
  
  Layout_V * layout = new Layout_V(this);
  
  layout->add(_p->widget);
  layout->stretch(_p->widget);
  
  Layout_H * layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add_stretch();
  layout_h->add(_p->close);
  
  // Initialize.
  
  size(Vec2i(400, 400));

  // Callbacks.

  _p->close->sig.set(this, close_callback);
}

Info::~Info()
{}

void Info::show()
{
  dialog::Base::show();

  _p->close->take_focus();
}

void Info::close_callback(bool)
{
  hide();
}

struct About::_About
{
  std::auto_ptr<Fl_Image> image;
  button::Push * close;
};

About::About() :
  dialog::Base(label_about),
  _p(new _About)
{
  _p->image = std::auto_ptr<Fl_Image>(new Fl_Pixmap((const char **)durant_xpm));

  // Widgets.
  
  label::Image * image = new label::Image(_p->image.get());

  scroll::Widget * scroll = new scroll::Widget;
  label::Text * text = new label::Text(app::global()->about());
  text->align(layout::LEFT, layout::LEFT);
  
  _p->close = new button::Push(dialog::label_close);
    
  // Layout.
  
  Layout_V * layout;
  Layout_H * layout_h;
  Layout_V * layout_scroll;
  
  layout = new Layout_V(this);
  layout->add(image);
  layout->add(scroll);
  layout->stretch(scroll);
  layout_scroll = new Layout_V(scroll);
  layout_scroll->add(text);
  layout_scroll->stretch(text);

  layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add_stretch();
  layout_h->add(_p->close);
  
  // Initialize.
  
  size(size_hint());

  // Callbacks.

  _p->close->sig.set(this, close_callback);
}

About::~About()
{}

void About::show()
{
  _p->close->take_focus();

  dialog::Base::show();
}

void About::close_callback(bool)
{
  hide();
}

const Str
  label_info = "Information",
  label_about = "About";

Global::Global() :
  _info(new Info),
  _about(new About)
{}

Global::~Global()
{}

Info * Global::info() const { return _info.get(); }
About * Global::about() const { return _about.get(); }

}
}

