/*------------------------------------------------------------------------------
 lib/djv_ui/button_check.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/button.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/label.h>
#include <djv_ui/style.h>
#include <FL/Fl.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Check_Button.H>
#include <FL/fl_draw.H>

namespace djv_ui {
namespace button {

namespace {

class _Button : public Fl_Check_Button
{
public:

  _Button() : Fl_Check_Button(0, 0, 0, 0) {}

  int handle(int in)
  {
    switch (in)
    {
      case FL_SHORTCUT:
        if (! (shortcut() ? Fl::test_shortcut(shortcut()) : test_shortcut()))
          return 0;
        value(! value());
        do_callback();
        return 1;
    }
    return Fl_Check_Button::handle(in);
  }
};

}

struct Check::_Check
{
  mem::Buffer<char> label;
  _Button * widget;
};

Check::Check(const Str & in) :
  sig(this),
  _value(false),
  _p(new _Check)
{
  Fl_Group::add(_p->widget = new _Button);

  //text_font(style::global()->font_bold());

  size(layout::FIXED, layout::FIXED);
  align(layout::LEFT, layout::CENTER);
  
  label(in);
  
  _p->widget->callback(widget_callback, this);
}

Check::~Check()
{}

void Check::set(bool in)
{
  if (in == _value) return;
  
  _value = in;
  
  _p->widget->value(_value);
  sig.emit(_value);
}

bool Check::get() const { return _value; }

void Check::label(const Str & in)
{
  if (in == _label) return;
  
  _label = in;
  _label_tmp = " " + _label; // XXX Add spacing.
  
  const int size = _label_tmp.size() + 1;
  _p->label.size(size);
  str::cstr(_label_tmp, _p->label(), size);
  _p->widget->label(_p->label());
}

const Str & Check::label() const { return _label; }

void Check::dirty()
{
  widget::Widget::dirty();

  _p->widget->labelfont(text_font());
  _p->widget->labelsize(style::global()->font_size());
  _p->widget->labelcolor(text_color());
  
  fl_font(text_font(), style::global()->font_size());
  const Vec2i label_size(_p->label() ? base::str_size(_p->label()) :
    Vec2i(fl_height()));

  size_hint(Vec2i(fl_height() + label_size.x, label_size.y) +
    style::global()->margin_text() * 2);
}

void Check::geom(const Box2i & in)
{
  widget::Widget::geom(in);
  _p->widget->resize(in.x, in.y, in.w, in.h);
}

const Box2i & Check::geom() const { return widget::Widget::geom(); }

void Check::widget_callback() { set(_p->widget->value() ? true : false); }

}
}
