/*------------------------------------------------------------------------------
 lib/djv_ui/dialog_color.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/button.h>
#include <djv_ui/color.h>
#include <djv_ui/dialog.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/style.h>

namespace djv_ui {
namespace dialog {

struct Color::_Color
{
  color::Widget * widget;
  color::Swatch * swatch;
  button::Push * close;
};

Color::Color(const djv_image::Color & value) :
  Base(label_color),
  sig(this),
  _p(new _Color)
{
  // Widgets.

  _p->widget = new color::Widget;
  
  _p->swatch = new color::Swatch;
  _p->swatch->size(layout::STRETCH, layout::STRETCH);
  
  _p->close = new button::Push(label_close);

  // Layout.

  Layout_V * layout = new Layout_V(this);
  
  Layout_H * layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout->stretch(layout_h);
  layout_h->add(_p->swatch);
  layout_h->add(_p->widget);
  layout_h->stretch(_p->widget);
  
  layout->add_spacer(-1, true);
  
  layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add_stretch();
  layout_h->add(_p->close);
  
  // Initialize.

  size(size_hint());
  
  // Callbacks.

  _p->widget->sig.set(this, widget_callback);
  _p->close->sig.set(this, close_callback);
}

Color::~Color()
{}

void Color::set(const djv_image::Color & in)
{
  if (in == _value) return;
  
  _value = in;
  
  widget_update();
  sig.emit(_value);
}

const djv_image::Color & Color::get() const { return _value; }

void Color::widget_callback(const djv_image::Color & in)
{
  set(in);
}

void Color::close_callback(bool)
{
  hide();
}

void Color::widget_update()
{
  callbacks(false);
  
  _p->widget->set(_value);
  _p->swatch->set(_value);
  
  callbacks(true);
}

}
}

