/*------------------------------------------------------------------------------
 lib/djv_ui/file_browser_fnc.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "file_browser.h"
#include <djv_ui/style.h>

namespace djv_ui {
namespace file_browser {

List<int> columns_size_list()
{
  return List<int>() <<
    0 <<
    (base::str_width("000.0000") + style::global()->margin() * 2) <<
#if ! defined(WIN32)
    (base::str_width("000000") + style::global()->margin() * 2) <<
#endif
    (
      base::str_width("000 000 00 00:00:00 0000") + 16 +
      style::global()->margin() * 2
    );
}

DJV_STR_OPERATOR_SET(IMAGE, label_image())
DJV_STR_OPERATOR_GET(IMAGE, label_image())
DJV_STR_OPERATOR_SET(SORT, label_sort())
DJV_STR_OPERATOR_GET(SORT, label_sort())

const List<Str> & label_image()
{
  static const List<Str> data = List<Str>() <<
    "Off" <<
    "Low Quality" <<
    "High Quality";
  return data;
}

const List<Str> & label_sort()
{
  static const List<Str> data = List<Str>() <<
    djv_base::file::label_sort()[djv_base::file::SORT_NAME] <<
    djv_base::file::label_sort()[djv_base::file::SORT_SIZE] <<
#if ! defined(WIN32)
    djv_base::file::label_sort()[djv_base::file::SORT_USER] <<
#endif
    djv_base::file::label_sort()[djv_base::file::SORT_TIME];
  return data;
}

const List<Str> & label_shortcut()
{
  static const List<Str> data = List<Str>() <<
    "Up" <<
    "Previous" <<
    "Current" <<
    "Home" <<
    "Desktop" <<
    "Root" <<
    "Reload" <<
    "Sequence Off" <<
    "Sequence Sparse" <<
    "Sequence Range" <<
    "Hidden" <<
    "Sort Name" <<
    "Sort Size" <<
#if ! defined(WIN32)
    "Sort User" <<
#endif
    "Sort Time" <<
    "Sort Reverse" <<
    "Sort Directories" <<
    "Bookmark Add" <<
    "Bookmark 1" <<
    "Bookmark 2" <<
    "Bookmark 3" <<
    "Bookmark 4" <<
    "Bookmark 5" <<
    "Bookmark 6" <<
    "Bookmark 7" <<
    "Bookmark 8" <<
    "Bookmark 9" <<
    "Bookmark 10" <<
    "Bookmark 11" <<
    "Bookmark 12";
  return data;
}

const Str
  dialog_title = "File Browser",
  dialog_bookmark_del = "Delete bookmark:",
  dialog_bookmark_del_all = "Delete all bookmarks?";

const Str
  menu_dir = "&Directory",
  menu_dir_up = "Up",
  menu_dir_prev = "Previous",
  menu_dir_recent = "Recent",
  menu_dir_current = "Current",
  menu_dir_home = "Home",
  menu_dir_desktop = "Desktop",
  menu_dir_drive = "Drives",
  menu_dir_root = "Root",
  menu_dir_reload = "Reload",
  menu_view = "&View",
  menu_view_image = "Image",
  menu_view_seq = "Sequence",
  menu_view_hidden = "Hidden",
  menu_view_sort = "Sort",
  menu_view_sort_reverse = "Sort Reverse",
  menu_view_sort_dir = "Sort Directories",
  menu_bookmark = "&Bookmarks",
  menu_bookmark_add = "Add",
  menu_bookmark_del = "Delete",
  menu_bookmark_del_all = "Delete All";

const Str
  tooltip_widget = "Selection.",
  tooltip_dir = "Current directory.",
  tooltip_up = "Up a directory.\nKeyboard shortcut: %%",
  tooltip_prev = "Previous directory.\nKeyboard shortcut: %%",
  tooltip_reload = "Reload current directory.\nKeyboard shortcut: %%";

const Str
  prefs_title = "File Browser";

namespace {

std::auto_ptr<Dialog> _dialog;
std::auto_ptr<Prefs> _prefs;

}

void set(Callback_Base * in, Signal<const File &>::Callback * callback)
{
  Dialog * dialog = global();
  dialog->sig.del();
  dialog->sig.set(in, callback);
  dialog->show();
}

void del(Callback_Base * in)
{
  Dialog * dialog = global();
  dialog->sig.del(in);
}

}
}

