/*------------------------------------------------------------------------------
 lib/djv_ui/file_browser_item.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include "file_browser.h"
#include <djv_ui/app.h>
#include <djv_ui/image.h>
#include <djv_image/image.h>

namespace djv_ui {
namespace file_browser {

Item::Item() :
  _prev(0),
  _next(0),
  _odd(false),
  _image(0),
  _image_valid(false),
  _image_init(false)
{}

Item::~Item()
{
  if (_image) delete _image;
}

void Item::set(const File & in, Item * prev, Item * next, bool odd)
{
  _value = in;
  _prev = prev;
  _next = next;
  _odd = odd;
  _image_valid = djv_image::io::global()->valid(_value);
}

namespace {

Vec2i thumbnail(const Vec2i & in, int size, IMAGE image,
  djv_image::data::PROXY * proxy)
{
  using namespace djv_image;
  
  const int image_size = math::max(in.x, in.y);
  if (image_size <= 0) return Vec2i();

  int _proxy = 0;
  double proxy_scale = double(data::proxy_scale(data::PROXY(_proxy)));
  if (IMAGE_LOW == image)
    while (
      (image_size / proxy_scale) > size * 2 &&
      _proxy < data::_PROXY_SIZE
    )
      proxy_scale = double(data::proxy_scale(data::PROXY(++_proxy)));

  if (proxy)
    *proxy = data::PROXY(_proxy);
  
  const double scale = size / double(image_size / proxy_scale);
  return vec::ceil<double, int>(Vec2f(in) / proxy_scale * scale);
}

}

Fl_Image * Item::image(int size, IMAGE image) const
{
  if (_image_init) return _image;
  
  DJV_DBG_OBJ("Item::image");
  DJV_DBG_PRN("size = " << size);
  DJV_DBG_PRN("image = " << image);
  DJV_DBG_PRN("value = " << _value);

try
{
  Item * that = const_cast<Item *>(this);
  that->_image_init = true;

  if (_image_valid)
  {
    base::Cursor cursor(FL_CURSOR_WAIT);
    
    std::auto_ptr<djv_image::io::Load> load(djv_image::io::global()->load(
      _value, &that->_image_info, 0, &that->_image_time));
    DJV_DBG_PRN("info = " << _image_info);

    djv_image::data::PROXY proxy(djv_image::data::PROXY(0));
    const Vec2i _size = thumbnail(_image_info.size, size, image, &proxy);
    DJV_DBG_PRN("size = " << _size);
    DJV_DBG_PRN("proxy = " << proxy);

    const djv_image::Image * image = load->get(-1, 0, proxy);
    DJV_DBG_PRN("image = " << *image);
    
    that->_image = image::image_to_fltk(*image, _size, image->profile);
  }
}
catch (Error in)
{
  app::global()->error(in, _value.name());
}
  return _image;
}

}
}
