/*------------------------------------------------------------------------------
 lib/djv_ui/file_browser_prefs.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/app.h>
#include <djv_ui/group.h>
#include <djv_ui/file.h>
#include <djv_ui/file_browser.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/prefs.h>
#include <djv_ui/style.h>

namespace djv_ui {
namespace file_browser {

Prefs::Prefs() :
  _image(IMAGE_HIGH),
  _seq(djv_base::file::SEQ_RANGE),
  _hidden(false),
  _sort(file_browser::SORT_NAME),
  _sort_reverse(false),
  _sort_dir(true),
  _pin(false)
{
  using djv_ui::shortcut::Shortcut;
  
  djv_ui::prefs::Prefs prefs(prefs::prefs_global(), "file_browser");

  prefs::get(&prefs, "recent", &_recent);
  prefs::get(&prefs, "image", &_image);
  prefs::get(&prefs, "seq", &_seq);
  prefs::get(&prefs, "hidden", &_hidden);
  prefs::get(&prefs, "sort", &_sort);
  prefs::get(&prefs, "sort_reverse", &_sort_reverse);
  prefs::get(&prefs, "sort_dir", &_sort_dir);
  prefs::get(&prefs, "bookmark", &_bookmark);
  prefs::get(&prefs, "pin", &_pin);

  const List<Str> & label_shortcut = file_browser::label_shortcut();
  _shortcut += Shortcut(label_shortcut[UP], '\\');
  _shortcut += Shortcut(label_shortcut[PREV], FL_BackSpace);
  _shortcut += Shortcut(label_shortcut[CURRENT], '.');
  _shortcut += Shortcut(label_shortcut[HOME], ',');
  _shortcut += Shortcut(label_shortcut[DESKTOP], 'k');
  _shortcut += Shortcut(label_shortcut[ROOT], '/');
  _shortcut += Shortcut(label_shortcut[RELOAD], 'r');
  _shortcut += Shortcut(label_shortcut[SEQ_OFF], 'q');
  _shortcut += Shortcut(label_shortcut[SEQ_SPARSE], 'a');
  _shortcut += Shortcut(label_shortcut[SEQ_RANGE], 'z');
  _shortcut += Shortcut(label_shortcut[HIDDEN], 'n');
  _shortcut += Shortcut(label_shortcut[SORT_NAME], '1');
  _shortcut += Shortcut(label_shortcut[SORT_SIZE], '2');
#if ! defined(WIN32)
  _shortcut += Shortcut(label_shortcut[SORT_USER], '3');
  _shortcut += Shortcut(label_shortcut[SORT_TIME], '4');
#else
  _shortcut += Shortcut(label_shortcut[SORT_TIME], '3');
#endif
  _shortcut += Shortcut(label_shortcut[SORT_REVERSE], '`');
  _shortcut += Shortcut(label_shortcut[SORT_DIR], 'd');
  _shortcut += Shortcut(label_shortcut[BOOKMARK_ADD], 'b');
  _shortcut += Shortcut(label_shortcut[BOOKMARK_1], FL_F + 1);
  _shortcut += Shortcut(label_shortcut[BOOKMARK_2], FL_F + 2);
  _shortcut += Shortcut(label_shortcut[BOOKMARK_3], FL_F + 3);
  _shortcut += Shortcut(label_shortcut[BOOKMARK_4], FL_F + 4);
  _shortcut += Shortcut(label_shortcut[BOOKMARK_5], FL_F + 5);
  _shortcut += Shortcut(label_shortcut[BOOKMARK_6], FL_F + 6);
  _shortcut += Shortcut(label_shortcut[BOOKMARK_7], FL_F + 7);
  _shortcut += Shortcut(label_shortcut[BOOKMARK_8], FL_F + 8);
  _shortcut += Shortcut(label_shortcut[BOOKMARK_9], FL_F + 9);
  _shortcut += Shortcut(label_shortcut[BOOKMARK_10], FL_F + 10);
  _shortcut += Shortcut(label_shortcut[BOOKMARK_11], FL_F + 11);
  _shortcut += Shortcut(label_shortcut[BOOKMARK_12], FL_F + 12);
  
  djv_ui::prefs::Prefs prefs_shortcut(prefs, "shortcut");
  Str tmp;
  for (uint i = 0; i < _shortcut.size(); ++i)
    if (prefs_shortcut.get(_shortcut[i].name, &tmp))
      shortcut::serialize(&tmp, &_shortcut[i].value);
}

Prefs::~Prefs()
{
  djv_ui::prefs::Prefs prefs(prefs::prefs_global(), "file_browser");
  
  prefs::set(&prefs, "recent", _recent);
  prefs::set(&prefs, "image", _image);
  prefs::set(&prefs, "seq", _seq);
  prefs::set(&prefs, "hidden", _hidden);
  prefs::set(&prefs, "sort", _sort);
  prefs::set(&prefs, "sort_reverse", _sort_reverse);
  prefs::set(&prefs, "sort_dir", _sort_dir);
  prefs::set(&prefs, "bookmark", _bookmark);
  prefs::set(&prefs, "pin", _pin);
  
  djv_ui::prefs::Prefs prefs_shortcut(prefs, "shortcut");
  for (uint i = 0; i < _shortcut.size(); ++i)
    prefs_shortcut.set(
      _shortcut[i].name,
      shortcut::serialize(_shortcut[i].value));
}

void Prefs::recent(const List<Str> & in)
{
  _recent = in;
}

void Prefs::recent_add(const Str & in)
{
  djv_ui::file::recent(in, &_recent);
}

const List<Str> & Prefs::recent() const { return _recent; }

void Prefs::image(IMAGE in)
{
  _image = in;
}

IMAGE Prefs::image() const { return _image; }

void Prefs::seq(djv_base::file::SEQ_COMPRESS in)
{
  _seq = in;
}

djv_base::file::SEQ_COMPRESS Prefs::seq() const { return _seq; }

void Prefs::hidden(bool in)
{
  _hidden = in;
}

bool Prefs::hidden() const { return _hidden; }
  
void Prefs::sort(SORT in)
{
  _sort = in;
}

SORT Prefs::sort() const { return _sort; }

void Prefs::sort_reverse(bool in)
{
  _sort_reverse = in;
}

bool Prefs::sort_reverse() const { return _sort_reverse; }

void Prefs::sort_dir(bool in)
{
  _sort_dir = in;
}

bool Prefs::sort_dir() const { return _sort_dir; }

void Prefs::bookmark(const List<Str> & in)
{
  _bookmark = in;
}

void Prefs::bookmark_add(const Str & in)
{
  _bookmark += in;
}

const List<Str> & Prefs::bookmark() const { return _bookmark; }

void Prefs::pin(bool in)
{
  _pin = in;
}

bool Prefs::pin() const { return _pin; }

void Prefs::shortcut(const List<shortcut::Shortcut> & in)
{
  if (in == _shortcut) return;
  
  _shortcut = in;
  
  djv_ui::app::global()->window_update();
}

const List<shortcut::Shortcut> & Prefs::shortcut() const { return _shortcut; }

}
}

