/*------------------------------------------------------------------------------
 lib/djv_ui/image_view.cxx
 
 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

//#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/app.h>
#include <djv_ui/image_view.h>
#include <djv_image/image.h>

namespace djv_ui {
namespace image_view {

Widget::Widget() :
  sig(this),
  _data(0),
  _zoom(1.0)
{}

Widget::~Widget()
{}

void Widget::set(const djv_image::data::Data * in)
{
  _data = in;
}

const djv_image::data::Data * Widget::get() const { return _data; }

void Widget::view(const Vec2i & in)
{
  if (in == _view) return;
  
  _view = in;
  
  sig.emit(true);
}

const Vec2i & Widget::view() const { return _view; }

void Widget::zoom(double in)
{
  if (in == _zoom) return;
  
  DJV_DBG_OBJ("Widget::zoom");
  DJV_DBG_PRN("in = " << in);
  
  _zoom = in;
  
  sig.emit(true);
}

void Widget::zoom(double in, const Vec2i & focus)
{
  if (in == _zoom) return;
  
  DJV_DBG_OBJ("Widget::zoom");
  DJV_DBG_PRN("in = " << in);
  DJV_DBG_PRN("focus = " << focus);
  
  _view = focus + Vec2i(Vec2f(_view - focus) * (in / _zoom));
  _zoom = in;
  
  sig.emit(true);
}

double Widget::zoom() const { return _zoom; }

void Widget::view_zero()
{
  _view = Vec2i();
  _zoom = 1.0;
  
  sig.emit(true);
}

void Widget::view_center()
{
  view(Vec2f(geom().size) / 2.0 - bbox().size / 2.0);
}

void Widget::view_fit()
{
  if (! _data) return;
  
  DJV_DBG_OBJ("Widget::view_fit");
  
  const Box2i & geom = this->geom();
  DJV_DBG_PRN("geom = " << geom.size);
  
  _view.zero();
  _zoom = 1.0;
  const Box2f bbox = this->bbox();
  DJV_DBG_PRN("bbox = " << bbox);
  
  _zoom = vec::size_valid(bbox.size) ?
    math::min(geom.w / bbox.size.x, geom.h / bbox.size.y) : 1.0;
  DJV_DBG_PRN("zoom = " << _zoom);

  _view =
    Vec2f(geom.size) / 2.0 -
    bbox.size * _zoom / 2.0 -
    this->bbox().position;
  DJV_DBG_PRN("view = " << _view);
  
  sig.emit(true);
}

void Widget::option(const djv_image::gl_data::Option & in)
{
  _option = in;
}

const djv_image::gl_data::Option & Widget::option() const { return _option; }

void Widget::draw()
{
  gl::Widget::draw();

  if (! visible()) return;
    
  DJV_DBG_OBJ("Widget::draw");
  DJV_DBG_PRN("valid = " << valid());
  DJV_DBG_PRN("view = " << _view);
  DJV_DBG_PRN("zoom = " << _zoom);

  const Box2i & geom = this->geom();
  //DJV_DBG_PRN("geom = " << geom.size);
    
  //if (! valid())
  //{
    djv_image::gl::ortho(Vec2i(geom.w, geom.h));
    glViewport(0, 0, geom.w, geom.h);
  //}
  
  djv_image::Color background(djv_image::pixel::RGB_F32);
  djv_image::color::convert(_option.background, &background);
  glClearColor(
    background.get_f32(0),
    background.get_f32(1),
    background.get_f32(2),
    0.0);
  //glClearColor(0, 1, 0, 0);
  glClear(GL_COLOR_BUFFER_BIT);
  
  if (! _data) return;
  DJV_DBG_PRN("data = " << *_data);
  
  djv_image::gl_data::Option option = _option;
  option.xform.position += _view;
  option.xform.scale *= _zoom;  
  try { _gl.draw(*_data, option); }
  catch (Error in) { app::global()->error(in); }
}

Box2f Widget::bbox() const
{
  if (! _data) return Box2f();
  
  const djv_image::data::Info & info = _data->info();
  
  djv_image::gl_data::Xform xform = _option.xform;
  xform.position += _view;
  xform.scale *= _zoom;
  return
    djv_image::gl_data::xform_mtx(xform) *
    Box2f(_data->info().size * djv_image::data::proxy_scale(info.proxy));
}

}
}
