/*------------------------------------------------------------------------------
 lib/djv_ui/num.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include <djv_ui/button.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/num.h>
#include <FL/Fl.H>

namespace djv_ui {
namespace num {

Int_Base::Int_Base(int min, int max) :
  Base<int>( min, max)
{
  inc(1, 10);
}

Float_Base::Float_Base(double min, double max) :
  Base<double>(min, max),
  _precision(0)
{
  inc(0.1, 1.0);
}

void Float_Base::precision(int in)
{
  if (in == _precision) return;
  
  _precision = in;
  
  set(get());
  
  value_callback();
}

int Float_Base::precision() const { return _precision; }

struct Inc::_Inc
{
  button::Tool * inc, * dec;
};

Inc::Inc() :
  inc_sig(this),
  dec_sig(this),
  _p(new _Inc)
{
  // Widgets.
  
  _p->inc = new button::Tool("tool_inc", 0);
  _p->inc->type(button::Tool::REPEAT);
  _p->dec = new button::Tool("tool_dec", 0);
  _p->dec->type(button::Tool::REPEAT);
  
  // Layout.
  
  Item::size(layout::FIXED, layout::STRETCH);
  
  Layout_V * layout = new Layout_V(this);
  layout->spacing(0);
  layout->margin(0);
  _p->inc->Item::size(layout::FIXED, layout::STRETCH);
  layout->add(_p->inc);
  _p->dec->Item::size(layout::FIXED, layout::STRETCH);
  layout->add(_p->dec);

  // Callbacks.
  
  _p->inc->sig.set(this, inc_callback);
  _p->dec->sig.set(this, dec_callback);
}

Inc::~Inc()
{}

void Inc::inc_callback(bool) { inc_sig.emit(true); }
void Inc::dec_callback(bool) { dec_sig.emit(true); }

}
}

