/*------------------------------------------------------------------------------
 lib/djv_ui/prefs_dialog.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/button.h>
#include <djv_ui/file_browser.h>
#include <djv_ui/group.h>
#include <djv_ui/image.h>
#include <djv_ui/image_io.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/prefs_dialog.h>
#include <djv_ui/shortcut.h>
#include <djv_ui/style.h>
#include <djv_ui/tab.h>

namespace djv_ui {
namespace prefs_dialog {

struct Dialog::_Dialog
{
  tab::Widget * tab;
  tab::Widget * shortcut_tab;
  button::Push * close;
};

Dialog::Dialog() :
  dialog::Base(dialog_title),
  _p(new _Dialog)
{
  // Widgets.

  _p->tab = new tab::Widget;
  _p->shortcut_tab = new tab::Widget;
  _p->close = new button::Push(dialog::label_close);

  group::Frame * frame = new group::Frame;
  frame->frame(FL_THIN_DOWN_FRAME);

  // Layout.

  Layout_V * layout = new Layout_V(this);
  
  Layout_V * layout_v = new Layout_V(layout);
  layout->stretch(layout_v);
  layout_v->margin(0);
  layout_v->add(frame);
  layout_v = new Layout_V(frame);
  layout_v->margin(0);
  layout_v->add(_p->tab);
  
  Layout_H * layout_h = new Layout_H(layout);
  layout_h->margin(0);
  layout_h->add_stretch();
  layout_h->add(_p->close);
  
  // Initialize.

  add(prefs_title, new prefs_dialog::Widget);
  add(image_io::prefs_save_title, new image_io::Prefs_Save);
  add(image_io::prefs_load_title, new image_io::Prefs_Load);
  add(shortcut::prefs_title, _p->shortcut_tab);
  add_shortcut(file_browser::prefs_title, new file_browser::Prefs_Widget);
  
  size(Vec2i(500, 400));

  // Callbacks.

  _p->close->sig.set(this, close_callback);
}

Dialog::~Dialog()
{}

void Dialog::add(const Str & name, widget::Base * in)
{
  DJV_DBG_OBJ("Dialog::add");
  DJV_DBG_PRN("name = " << name);
  
  _p->tab->tab_insert(name, in);
}

void Dialog::add_shortcut(const Str & name, widget::Base * in)
{
  DJV_DBG_OBJ("Dialog::add_shortcut");
  DJV_DBG_PRN("name = " << name);
  
  _p->shortcut_tab->tab_insert(name, in);
}

void Dialog::show()
{
  _p->close->take_focus();

  dialog::Base::show();
}

void Dialog::close_callback(bool)
{
  hide();
}

}
}

