/*------------------------------------------------------------------------------
 lib/djv_ui/scroll.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#undef DJV_DBG
//#define DJV_DBG

#include <djv_ui/layout_row.h>
#include <djv_ui/scroll.h>
#include <djv_ui/style.h>
#include <FL/Fl.H>
#include <FL/Fl_Box.H>

namespace djv_ui {
namespace scroll {

Widget::Widget() :
  Fl_Scroll(0, 0, 0, 0),
  _resizable(true)
{
  Fl_Scroll::end();
  //Fl_Scroll::box(FL_NO_BOX);
  Fl_Scroll::box(FL_THIN_DOWN_FRAME);
  //Fl_Scroll::type(Fl_Scroll::BOTH_ALWAYS);
  //Fl_Scroll::scrollbar.slider(FL_THIN_UP_BOX);
  //Fl_Scroll::hscrollbar.slider(FL_THIN_UP_BOX);
  
  add(_dummy_widget = new Fl_Box(0, 0, 0, 0));
  _dummy_widget->box(FL_NO_BOX);
  _dummy_widget->hide();
  
  //Layout_V * layout = new Layout_V(this);
  //layout->margin(0);
  new Layout_V(this);
}

Widget::~Widget()
{}

int Widget::handle(int in)
{
  switch (widget::Base::handle(in))
  {
    case -1: return 0;
    case 1: return 1;
  }
  return Fl_Scroll::handle(in);
}

void Widget::show()
{
  Fl_Scroll::show();
  widget::Base::show();
}

void Widget::draw()
{
  Fl_Scroll::draw();
  widget::Base::draw();
}

void Widget::resizable(bool in)
{
  _resizable = in;
}

bool Widget::resizable() const { return _resizable; }

void Widget::dirty()
{
  DJV_DBG_OBJ("Widget::dirty");
  
  widget::Base::dirty();

  size_hint(Vec2i(Fl_Scroll::scrollbar.w(), Fl_Scroll::hscrollbar.h()) +
    1 * 2);

  _size_min = style::global()->size_text();
}

void Widget::geom(const Box2i & in)
{
  DJV_DBG_OBJ("Widget::geom");
  DJV_DBG_PRN("in = " << in);

  Fl_Scroll::resize(in.x, in.y, in.w, in.h);
  Item::geom(in);

  const Vec2i & size_hint = layout()->size_hint();
  DJV_DBG_PRN("size hint = " << size_hint);

  const Vec2i scroll(Fl_Scroll::xposition(), Fl_Scroll::yposition());
  DJV_DBG_PRN("scroll = " << scroll);
  
  Vec2i size;
  if (_resizable)
  {
    size = in.size + scroll;
    
    // XXX Scrollbar visibility.

    //size -= Vec2i(Fl_Scroll::scrollbar.w(), Fl_Scroll::hscrollbar.h());
    
    if (scroll.y || size_hint.y > size.y)
      size.x -= Fl_Scroll::scrollbar.w();
    if (scroll.x || size_hint.x > size.x)
      size.y -= Fl_Scroll::hscrollbar.h();
  }
  else
  {
    size = size_hint;
  }

  Box2i geom(in.position - scroll, size);
  
  // XXX Frame border.
  
  geom = box::border(geom, Vec2i(-1));

  _dummy_widget->resize(geom.x, geom.y, geom.w, geom.h);
  layout()->geom(geom);
}

const Box2i & Widget::geom() const { return widget::Base::geom(); }

void Widget::resize(int x, int y, int w, int h) { geom(Box2i(x, y, w, h)); }

const Vec2i & Widget::size_min() const { return _size_min; }

int Widget::visible() const { return Fl_Scroll::visible(); }

}
}

