/*------------------------------------------------------------------------------
 plugin/djv_quicktime_unix/ui.cxx

 Copyright (c) 2004-2008 Darby Johnston. All rights reserved.
 Use is subject to license terms.
------------------------------------------------------------------------------*/

#include "ui.h"
#include <djv_ui/choice.h>
#include <djv_ui/group.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/label.h>
#include <djv_ui/style.h>

namespace djv_quicktime {
namespace ui {

extern "C" {
DJV_PLUGIN_EXPORT plugin::Plugin * djv_ui_image_save() { return new Save; }
}

struct Save::_Save
{
  choice::Widget * codec;
};

Save::Save() :
  _plugin(0),
  _p(new _Save)
{
  // Widgets.
  
  group::Box * codec_group =
    new group::Box(label_codec_group);
  _p->codec =
    new choice::Widget(djv_quicktime::label_codec(), label_codec);

  // Layout.
  
  layout::group(List<layout::Item *>() <<
    _p->codec);
  
  Layout_V * layout, * layout_group;
  
  layout = new Layout_V(this);

  layout->add(codec_group);
  layout_group = new Layout_V(codec_group);
  layout_group->spacing(style::global()->spacing_large());
  layout_group->add(_p->codec);
  
  layout->add_stretch();
  
  // Initialize.
  
  plugin_update();
  widget_update();
  
  // Callbacks.
  
  _p->codec->sig.set(this, codec_callback);
}

plugin::Plugin * Save::copy() const { return new Save; }

const Str & Save::name() const { return djv_quicktime::name; }

void Save::set(djv_image::io::Plugin * in)
{
  _plugin = in;
  
  widget_update();
}

void Save::codec_callback(int in)
{
  if (! _plugin) return;
  
  _option.codec = djv_quicktime::label_codec()[in];
  
  plugin_update();
  sig.emit(_plugin);
}

void Save::plugin_update()
{
  callbacks(false);

  if (_plugin)
  {
    Str tmp;
    tmp << _option.codec;
    _plugin->option(_plugin->option_list()[io::SAVE_CODEC], &tmp);
  }

  callbacks(true);
}

void Save::widget_update()
{
  callbacks(false);

try
{
  if (_plugin)
  {
    Str tmp;
    tmp = _plugin->option(_plugin->option_list()[io::SAVE_CODEC]);
    tmp >> _option.codec;
  }
}
catch (Str) {}

  _p->codec->set(list::find(_option.codec, djv_quicktime::label_codec()));
  
  callbacks(true);
}

const Str
  label_codec_group = "Codec",
  label_codec = "Value:";

}
}

