//#define DJV_DBG

#include "audio.h"
#include <djv_ui/layout_row.h>

namespace audio {

App::App(int argc, char ** argv) :
  djv_ui::app::App("audio", argc, argv)
{
  DJV_DBG_OBJ("App::App");
  
  // Command line.
  
  if (argc < 2)
  {
    print("usage: test_audio (input)...");
    djv_image::app::App::exit(djv_base::app::EXIT_ERROR);
    return;
  }

  // Initialize.
  
  try { _audio_factory.init(); }
  catch (Error in) { error(in); }
  
  valid(true);
  
  for (int i = 1; i < argc; ++i)
    window_add(argv[i]);
}

App::~App()
{
  DJV_DBG_OBJ("App::~App");

  Fl::remove_timeout(idle_callback, this);
  
  DJV_DBG_PRN("window = " << int(_window.size()));
  for (uint i = 0; i < _window.size(); ++i)
    _window[i]->del();
}

void App::window_add(const Str & in)
{
  DJV_DBG_OBJ("App::window_add");
  DJV_DBG_PRN("in = " << in);
  
  _window += new Window(in);
}

void App::window_del(Window * in)
{
  DJV_DBG_OBJ("App::window_del");

  idle_del(in);
  in->del();
  _window.erase(std::find(_window.begin(), _window.end(), in));
}

const double idle_timeout = 0.001;
  
void App::idle_add(Window * in)
{
  DJV_DBG_OBJ("App::idle_add");
  
  _idle += in;
  if (1 == _idle.size())
    Fl::add_timeout(idle_timeout, idle_callback, this);
}

void App::idle_del(Window * in)
{
  DJV_DBG_OBJ("App::idle_del");

  List<Window *>::iterator i = std::find(_idle.begin(), _idle.end(), in);
  if (i != _idle.end()) _idle.erase(i);
  if (! _idle.size()) Fl::remove_timeout(idle_callback, this);
}

void App::idle_callback()
{
  DJV_DBG_OBJ("App::idle_callback");
  
  for (uint i = 0; i < _idle.size(); ++i)
    _idle[i]->idle();
  Fl::add_timeout(idle_timeout, idle_callback, this);
}

}

int main(int argc, char ** argv) { return audio::App(argc, argv).exec(); }
