#include <djv_base/file.h>
#include <djv_image/io.h>
#include <djv_ui/layout_row.h>
#include <djv_ui/window.h>
#include <djv_ui/image_view.h>
#include <djv_ui/app.h>
#include <FL/Fl.H>

class Widget : public djv_ui::image_view::Widget
{
public:

  int handle(int in)
  {
    const djv_base::Vec2i mouse(Fl::event_x(), geom().h - 1 - Fl::event_y());
    switch (in)
    {
      case FL_SHORTCUT:
        switch (Fl::event_key())
        {
          case '-': zoom(zoom() * 0.5, mouse); redraw(); return 1;
          case '=': zoom(zoom() * 2.0, mouse); redraw(); return 1;
          case FL_BackSpace: view_zero(); redraw(); return 1;
        }
        break;
      
      case FL_PUSH:
        _view_tmp = view();
        _mouse_push = mouse;
        return 1;
      
      case FL_DRAG:
        view(_view_tmp + mouse - _mouse_push);
        redraw();
        return 1;
    }
    return djv_ui::image_view::Widget::handle(in);
  }
  
private:

  djv_base::Vec2i _view_tmp, _mouse_push;
};

class App : public djv_ui::app::App
{
public:

  App(int argc, char ** argv);
  ~App();
  
  void update();

private:

  DJV_CALLBACK(App, exit_callback, bool) { exit(); }

  djv_base::File _file;
  djv_image::io::Load * _load;
  djv_image::Info _info;
  djv_image::Time _time;
  djv_ui::window::Widget * _window;
  Widget * _widget;
  int _frame;
};

void update_callback(void *);

App::App(int argc, char ** argv) :
  djv_ui::app::App("image_play", argc, argv),
  _load(0),
  _window(0),
  _frame(0)
{
  using namespace djv_base;
  
  /*djv_image::gl::fixed(true);
  djv_image::gl_data::texture_tile(Vec2i(64));
  djv_image::gl_data::texture_dbg(true);*/
  
  // Command line.
  
  if (argc != 2)
  {
    print("usage: test_image_play (input)");
    djv_image::app::App::exit(app::EXIT_ERROR);
    return;
  }
  
  _file.set(argv[1]);
  if (_file.seq_valid()) _file.type = file::SEQ;
  try
  {
    _load = djv_image::io::global()->load(_file, &_info, 0, &_time);
  }
  catch (Error error)
  {
    this->error(error, _file.name());
    djv_image::app::App::exit(app::EXIT_ERROR);
    return;
  }

  valid(true);
  
  // Widgets.
  
  _window = new djv_ui::window::Widget(name());
  
  _widget = new Widget;
  
  // Layout.
  
  djv_ui::Layout_V * layout = new djv_ui::Layout_V(_window);
  layout->margin(0);
  layout->add(_widget);
  layout->stretch(_widget);
  
  // Initialize.

  //update();
  Fl::add_idle(update_callback, this);

  //_widget->zoom(0.5);
  
  _window->size(_info.size);
  _window->show();

  // Callbacks.
  
  _window->close_sig.set(this, exit_callback);
}

App::~App()
{
  if (_window) _window->del();
  if (_load) delete _load;
}

void App::update()
{
  using namespace djv_base;
  
  static djv_base::time::Timer t;
  static double average = 0.0;
  static int accum = 0;
  t.check();
  const double fps = t.fps();
  t.start();
  if (fps < 1000.0)
  {
    average += fps;
    ++accum;
  }
  print(str::Format("FPS = %% (%%)").arg(fps).arg(average / double(accum)));
  
  const djv_image::Image * image = 0;
  try { image = _load->get(_time.list[_frame], 0, 0); }
  catch (Error) {}
  if (image)
  {
    _widget->set(image);
    djv_image::gl_data::Option option;
    option.profile = image->profile;
    _widget->option(option);
    _widget->redraw();
  }

  ++_frame;
  if (_frame >= int(_time.list.size())) _frame = 0;
}

void update_callback(void * in) { reinterpret_cast<App *>(in)->update(); }

int main(int argc, char ** argv) { return App(argc, argv).exec(); }

